package cn.com.qmth.ac.service.impl;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import cn.com.qmth.ac.bean.RedirectParam;
import cn.com.qmth.ac.bean.StatusException;
import cn.com.qmth.ac.bean.SystemConstant;
import cn.com.qmth.ac.service.CommonService;
import cn.com.qmth.ac.util.ByteUtil;
import cn.com.qmth.ac.util.HttpUtil;
import cn.com.qmth.ac.util.SHA256;
import cn.com.qmth.ac.util.ServletUtil;

@Service
public class CommonServiceImpl implements CommonService {

    private final static Logger log = LoggerFactory.getLogger(CommonServiceImpl.class);

    @Autowired
    private Environment environment;

    @Override
    public void redirect(RedirectParam param) {

        HttpServletResponse response = ServletUtil.getResponse();
        response.setHeader(SystemConstant.ACCESS_CONTROL_ALLOW_ORIGIN, SystemConstant.PATH_MATCH);

        try {
            if (param.getTicketValid()) {
                long timestamp = System.currentTimeMillis();
                String secretKey = param.getSecretKey();
                StringBuilder joinStr = new StringBuilder().append(param.getRootOrgId()).append(param.getAppId())
                        .append(timestamp).append(secretKey);
                byte[] bytes = SHA256.encode(joinStr.toString());
                String token = ByteUtil.toHexAscii(bytes);
                String url = param.getReturnUrl() + "/#/access?loginName=" + param.getLoginName() + "&orgId="
                        + param.getRootOrgId() + "&appId=" + param.getAppId() + "&timestamp=" + timestamp + "&token="
                        + token + "&from=cas";
                response.sendRedirect(url);
            } else {
                response.sendRedirect(param.getReturnUrl());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public RedirectParam authentication(String schoolCode) {
        HttpServletRequest request = ServletUtil.getRequest();
        String authserver = environment.getProperty("examcloud.cas.authserver." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(authserver)) {
            throw new StatusException(schoolCode + "未配置cas.authserver");
        }
        String authlogin = environment.getProperty("examcloud.cas.authlogin." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(authserver)) {
            throw new StatusException(schoolCode + "未配置cas.authlogin");
        }

        String authlogout = environment.getProperty("examcloud.cas.authlogout." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(authlogout)) {
            throw new StatusException(schoolCode + "未配置cas.authlogout");
        }

        String ecLogin = environment.getProperty("examcloud.login." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(ecLogin)) {
            throw new StatusException(schoolCode + "未配置examcloud.login");
        }

        String appId = environment.getProperty("examcloud.app-id." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(appId)) {
            throw new StatusException(schoolCode + "未配置examcloud.app-id");
        }

        String rootOrgId = environment.getProperty("examcloud.root-org-id." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(rootOrgId)) {
            throw new StatusException(schoolCode + "未配置examcloud.root-org-id");
        }
        String secretKey = environment.getProperty("examcloud.secret-key." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(secretKey)) {
            throw new StatusException(schoolCode + "未配置examcloud.login");
        }

        RedirectParam rp = new RedirectParam();

        rp.setAppId(appId);
        rp.setRootOrgId(rootOrgId);
        rp.setSecretKey(secretKey);

        String tkt = request.getParameter("ticket");
        if (Objects.isNull(tkt) || tkt.length() == 0) {
            rp.setTicketValid(false);
            rp.setReturnUrl(authlogin);
        } else {// 校验ticket，获取用户信息
            // 发送请求
            Map<String, Object> map = new LinkedHashMap<>();
            map.put("ticket", tkt);
            map.put(SystemConstant.SERVICE_KEY, ecLogin);
            try {
                String data = HttpUtil.get(authserver, map, null, System.currentTimeMillis());
                log.warn("data:{}", data);
                if (data == null) {
                    rp.setTicketValid(false);
                    rp.setReturnUrl(authlogin);
                } else {
                    String loginName = getLoginName(data);
                    if (StringUtils.isBlank(loginName)) {
                        rp.setTicketValid(false);
                        rp.setReturnUrl(authlogin);
                    } else {
                        rp.setTicketValid(true);
                        rp.setLoginName(loginName);
                        rp.setReturnUrl(ecLogin);
                    }
                }
            } catch (IOException e) {
                throw new StatusException("ticket校验出错", e);
            }
        }
        return rp;
    }

    private String getLoginName(String data) {
        Document doc = Jsoup.parse(data);
        Elements user = doc.getElementsByTag("cas:user");
        if (user == null || user.size() != 1) {
            return null;
        }
        return user.get(0).text().trim();
    }

    @Override
    public void logout(String schoolCode) {
        HttpServletResponse response = ServletUtil.getResponse();

        response.setHeader(SystemConstant.ACCESS_CONTROL_ALLOW_ORIGIN, SystemConstant.PATH_MATCH);
        try {
            response.sendRedirect(getLogoutUrl(schoolCode));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private String getLogoutUrl(String schoolCode) {
        String authlogout = environment.getProperty("examcloud.cas.authlogout." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(authlogout)) {
            throw new StatusException(schoolCode + "未配置cas.authlogout");
        }

        String host = environment.getProperty("examcloud.auth.center.host");
        if (StringUtils.isBlank(host)) {
            throw new StatusException("未配置examcloud.auth.center.host");
        }

        try {
            String loginUrlRoot = host + "/api/auth/login/" + schoolCode;
            String encodeLoginUrlRoot = URLEncoder.encode(loginUrlRoot, StandardCharsets.UTF_8.toString());
            return authlogout + SystemConstant.GET_UNKNOWN + SystemConstant.SERVICE_KEY + SystemConstant.GET_EQUAL
                    + encodeLoginUrlRoot;
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }
}
